function [C,c] = build_flow_constraint(M, dec, X, normal, area, corner_angle, harmonic_forms, Omega, if_area_const, idvx_lock)

% omega is a closed form
C1 = kron(eye(3,3), dec.d1d)*Omega;

if M.is_bnd
    error('This code does not support mesh with boundaries (yet)');
end

% Cyles constraints
if ~isempty(harmonic_forms)
    % omega is orthogonal to hormonic forms
    C2 = kron(eye(3,3), harmonic_forms')*Omega;

    % Immersion differential is orthogonal to harmonic forms
    [vf_face1,vf_face2,vf_face3] = primal_harmonic_form_vector_field(M, X, area, harmonic_forms);
    C3 = function_preservation_constraints(vf_face1, vf_face2, vf_face3, M, dec, area, Omega);
else
    C2 = sparse(0, M.nv+M.ne);
    C3 = sparse(0, M.nv+M.ne);
end

% Area constraint
if if_area_const
    C_area = [diag(dec.star0p)', sparse(1,M.ne)];
else
    C_area = sparse(0, M.nv+M.ne);
end

% Position constraint
if exist('idvx_lock','var') && ~isempty(idvx_lock)
    C_pos = point_constraints(idvx_lock, M, dec, X, normal, area, Omega);
else
    C_pos = sparse(0, M.nv+M.ne);
end

C = [C1; C2; C3; C_area; C_pos];
c = zeros(size(C,1),1);