function [Xp,dX] = build_surface(M, X, dec, u, e1, e2, e3)

% Integration on edges
edge = X(M.E2V(:,1),:) - X(M.E2V(:,2),:);

% Edge deformation
expu = exp((u(M.E2V(:,1)) + u(M.E2V(:,2)))/2);
e1_edge = (e1(M.E2T2(:,1),:) + e1(M.E2T2(:,2),:))/2;
e2_edge = (e2(M.E2T2(:,1),:) + e2(M.E2T2(:,2),:))/2;
e3_edge = (e3(M.E2T2(:,1),:) + e3(M.E2T2(:,2),:))/2;

dX = expu.*[dot(e1_edge, edge, 2), dot(e2_edge, edge, 2), dot(e3_edge, edge, 2)];

% Integration with/out seamless constraints
W = dec.W + 1e-8*dec.star0p;
W = (W + W')/2;
div_dX = dec.d1d*dec.star1p*dX;
Xp = W\div_dX;

% Rescale to unit area
Xp = recenter_mesh(Xp, M.T);

% Find best alignment with previous mesh
Xp = best_rotation(X, Xp);
end

function [Xp,R] = best_rotation(Xr, X)
    [U,S,V] = svd(X'*Xr);
    R = U*V';
    if det(R) < 0
        [~,id] = min(diag(S));
        s = ones(3,1);
        s(id) =-1;
        R = U*diag(s)*V';
    end
    
    Xp = X*R;
end
