function [e1,e2,e3,q] = compute_rotation_field(M, omega)

omega(M.ide_bound,:) = 0;

% Convert omega into quaternion exponential
Rot = axis_angle_to_quaternion_mult(omega);

% Build matrix of quaternion difference
R = matrix_vector_multiplication(Rot);
I1 = sparse(1:M.ne, M.E2T2(:,1), 1, M.ne, M.nf);
I2 = sparse(1:M.ne, M.E2T2(:,2), 1, M.ne, M.nf);
D0d = R*kron(eye(4,4), I1) - kron(eye(4,4), I2);

% Build 'Laplacian for quaternion
Wrot = D0d'*D0d;
Wrot = (Wrot + Wrot')/2;

% Find best quaternion
[x,ev] = eigs(Wrot, 1, 'sm');
q = reshape(x, [M.nf,4]);

% Project to unit quaternion
q = q./sqrt(sum(q.^2,2));

% Convert to rotation matrix
[e1,e2,e3] = quaternion_to_rotation(q);

end

function Q = axis_angle_to_quaternion_mult(omega)
th = sqrt(sum(omega.^2,2));
ax = omega./th;
ax(th < 1e-5,:) = 0;
q = [cos(th/2), ax.*sin(th/2)];

Q = [ q(:,1),-q(:,2),-q(:,3),-q(:,4), ...
      q(:,2), q(:,1),-q(:,4), q(:,3), ...
      q(:,3), q(:,4), q(:,1),-q(:,2), ...
      q(:,4),-q(:,3), q(:,2), q(:,1)];
end

function [e1,e2,e3] = quaternion_to_rotation(q)
M = quat2rotm(q);
E = reshape(M, [9,size(q,1)])';
e1 = E(:,1:3);
e2 = E(:,4:6);
e3 = E(:,7:9);

% e1 = [q(:,1).^2 + q(:,2).^2 - q(:,3).^2 - q(:,4).^2, 2*q(:,2).*q(:,3) + 2*q(:,1).*q(:,4)          , 2*q(:,2).*q(:,4) - 2*q(:,1).*q(:,3)          ];
% e2 = [2*q(:,2).*q(:,3) - 2*q(:,1).*q(:,4)          , q(:,1).^2 - q(:,2).^2 + q(:,3).^2 - q(:,4).^2, 2*q(:,3).*q(:,4) + 2*q(:,1).*q(:,2)          ];
% e3 = [2*q(:,2).*q(:,4) + 2*q(:,1).*q(:,3)          , 2*q(:,3).*q(:,4) - 2*q(:,1).*q(:,2)          , q(:,1).^2 - q(:,2).^2 - q(:,3).^2 + q(:,4).^2];
end

function M = matrix_vector_multiplication(A)
% A = [A1 A2 A3]
%     [A4 A5 A6]
%     [A7 A8 A9]
% M * B = A*B

nv = size(A,1);
n = sqrt(size(A,2));

I = repmat((1:nv)', [n,n]) + repelem((0:n-1)*nv, n*nv,1);
J = repmat((1:nv)', [n,n]) + repelem(repmat((0:n-1)'*nv, [1,n]), nv,1);
S = reshape(A, [n*nv,n]);
M = sparse(I, J, S, n*nv, n*nv);
end