function [C,c] = function_preservation_constraints(vf_face1, vf_face2, vf_face3, M, dec, area, Omega)

% Create all possible right-hand side
nb_fct = size(vf_face1,2);

if nb_fct > 0
    % Average on vertices
    A = sparse(M.T, repmat((1:M.nf)', [1,3]), repmat(area/3,[1,3]), M.nv, M.nf);
    vf_vx1 = A*vf_face1;
    vf_vx2 = A*vf_face2;
    vf_vx3 = A*vf_face3;

    Cu = [[vf_vx1'; vf_vx2'; vf_vx3'], sparse(3*nb_fct, M.ne)];

    % Compute laplacian on primal 1-form
    % cf "Design of Tangent Vector Fields"
    W = dec.d1p'*dec.star2p*dec.d1p + dec.star1p*dec.d0p*dec.star2d*dec.d0p'*dec.star1p;
    W = (W + W')/2;

    % Compute solution of d a_i = vf_i
    sdz1 = W\(dec.d0d*vf_face1);
    sdz2 = W\(dec.d0d*vf_face2);
    sdz3 = W\(dec.d0d*vf_face3);

    % Assemble cross product
    Z = sparse(M.ne, nb_fct);
    Ch = [Z',-sdz3', sdz2'; sdz3', Z',-sdz1'; -sdz2', sdz1', Z']*Omega;
    
    C = Cu - Ch;
else
    C = sparse(0, M.nv+M.ne);
end

c = zeros(size(C,1),1);

