function Omega = omega_mat(M, X, normal, corner_angle)

cot_ang = cot(corner_angle);

% Dual length
w1 = 0.5*sign(M.T2E(:,1)).*cot_ang(:,3);
w2 = 0.5*sign(M.T2E(:,2)).*cot_ang(:,1);
w3 = 0.5*sign(M.T2E(:,3)).*cot_ang(:,2);

% Edge tangent and normals
E1 = X(M.T(:,2),:) - X(M.T(:,1),:);
E2 = X(M.T(:,3),:) - X(M.T(:,2),:);
E3 = X(M.T(:,1),:) - X(M.T(:,3),:);
E1 = w1.*E1;
E2 = w2.*E2;
E3 = w3.*E3;

B1 = cross(E1, normal, 2);
B2 = cross(E2, normal, 2);
B3 = cross(E3, normal, 2);

% Normal-normal component
I = abs(M.T2E(:,[1 1 1 2 2 2 3 3 3]));
I = [I; M.ne+I; 2*M.ne+I];
J = abs(M.T2E(:,[1 2 3 1 2 3 1 2 3]));
J = [J; J; J];
S = [  - cot_ang(:,1).*cot_ang(:,2), 1 - cot_ang(:,2).*cot_ang(:,3), 1 - cot_ang(:,3).*cot_ang(:,1), ...
     1 - cot_ang(:,1).*cot_ang(:,2),   - cot_ang(:,2).*cot_ang(:,3), 1 - cot_ang(:,3).*cot_ang(:,1), ...
     1 - cot_ang(:,1).*cot_ang(:,2), 1 - cot_ang(:,2).*cot_ang(:,3),   - cot_ang(:,3).*cot_ang(:,1)];
S = repmat(S, [3,1]).*repelem([E1(:),E2(:),E3(:)], 1,3);
nTn = sparse(I, J, S, 3*M.ne, M.ne);

% Tangent-normal component
I = abs(M.T2E(:,[1 1 1 2 2 2 3 3 3]));
I = [I; M.ne+I; 2*M.ne+I];
J = abs(M.T2E(:,[1 2 3 1 2 3 1 2 3]));
J = [J; J; J];
S = [-cot_ang(:,1)+cot_ang(:,2),-cot_ang(:,2)-cot_ang(:,3),+cot_ang(:,3)+cot_ang(:,1), ...
     +cot_ang(:,1)+cot_ang(:,2),-cot_ang(:,2)+cot_ang(:,3),-cot_ang(:,3)-cot_ang(:,1), ...
     -cot_ang(:,1)-cot_ang(:,2),+cot_ang(:,2)+cot_ang(:,3),-cot_ang(:,3)+cot_ang(:,1)]/2;
S = repmat(S, [3,1]).*repelem([B1(:),B2(:),B3(:)], 1,3);
tTn = sparse(I, J, S, 3*M.ne, M.ne);

% Differential u
I = abs(M.T2E(:,[1 1 2 2 3 3]));
I = [I; M.ne+I; 2*M.ne+I];
J = M.T(:,[1 2 2 3 3 1]);
J = [J; J; J];
S = [w1,-w1, w2,-w2, w3,-w3];
S = [S; S; S].*repmat(-normal(:), [1,6]);
sDu = sparse(I, J, S, 3*M.ne, M.nv);

% Assemble omega
Omega = [sDu, nTn + tTn];
