function [C,c] = point_constraints(idvx_fix, M, dec, X, normal, area, Omega)

% Create all possible right-hand side
n = length(idvx_fix);
nb_fct = round(n*(n-1)/2);

b = zeros(M.nv,nb_fct);
k = 1;
for i = 1:n
    for j = 1:i-1
        b(idvx_fix(i),k) = 1;
        b(idvx_fix(j),k) =-1;

        k = k + 1;
    end
end

% Find face gradient of solution of Poisson problem
g = dec.W\b;
[vf_face1,vf_face2,vf_face3] = face_gradient(M, X, normal, area, g);

% Assemble constraints
[C,c] = function_preservation_constraints(vf_face1, vf_face2, vf_face3, M, dec, area, Omega);

end

function [grad_g1,grad_g2,grad_g3] = face_gradient(M, X, normal, area, g)
edge12 = cross(normal, X(M.T(:,2),:) - X(M.T(:,1),:), 2)./(2*area);
edge23 = cross(normal, X(M.T(:,3),:) - X(M.T(:,2),:), 2)./(2*area);
edge31 = cross(normal, X(M.T(:,1),:) - X(M.T(:,3),:), 2)./(2*area);

g_tri1 = g(M.T(:,1),:);
g_tri2 = g(M.T(:,2),:);
g_tri3 = g(M.T(:,3),:);

grad_g1 = edge12(:,1).*g_tri3 + edge23(:,1).*g_tri1 + edge31(:,1).*g_tri2;
grad_g2 = edge12(:,2).*g_tri3 + edge23(:,2).*g_tri1 + edge31(:,2).*g_tri2;
grad_g3 = edge12(:,3).*g_tri3 + edge23(:,3).*g_tri1 + edge31(:,3).*g_tri2;
end
