function fig = plot_principal_direction_face(fig, X, T, S, e1r, e2r)
% Plot principal direction of an face-based symmetric tensor

nf = size(T,1);
if isempty(fig)
    fig = figure;
end

E1 = zeros(nf,3);
E2 = zeros(nf,3);
kappa = zeros(nf,2);
for i = 1:nf
    A = [S(i,1), S(i,2); S(i,2), S(i,3)];

    [V,D] = eig(A);
%     assert(norm(V*D*V' - A, 'fro') < 1e-6);

    kappa(i,:) = diag(D);
    [~,id] = max(kappa(i,:));
    if id == 2
        V = V(:,[2 1]);
        kappa(i,:) = [kappa(i,2), kappa(i,1)];
    end
    E1(i,:) = e1r(i,:).*V(1,1) + e2r(i,:).*V(2,1);
    E2(i,:) = e1r(i,:).*V(1,2) + e2r(i,:).*V(2,2);
end
col = abs(kappa(:,1) - kappa(:,2));

pt = (X(T(:,1),:) + X(T(:,2),:) + X(T(:,3),:))/3;

fig = figure(fig);
subplot(1,3,1);
trisurf(T, X(:,1), X(:,2), X(:,3), col, 'edgecolor', 'none');
hold on;
quiver3(pt(:,1), pt(:,2), pt(:,3), E1(:,1), E1(:,2), E1(:,3), 1, 'r', 'LineWidth',1, 'ShowArrowHead','off');
quiver3(pt(:,1), pt(:,2), pt(:,3),-E1(:,1),-E1(:,2),-E1(:,3), 1, 'r', 'LineWidth',1, 'ShowArrowHead','off');
quiver3(pt(:,1), pt(:,2), pt(:,3), E2(:,1), E2(:,2), E2(:,3), 1, 'b', 'LineWidth',1, 'ShowArrowHead','off');
quiver3(pt(:,1), pt(:,2), pt(:,3),-E2(:,1),-E2(:,2),-E2(:,3), 1, 'b', 'LineWidth',1, 'ShowArrowHead','off');
hold off;
axis equal; colorbar;
title('Curvature direction');

% Mean curvature
H = (S(:,1) + S(:,3))/2;
subplot(1,3,2);
trisurf(T, X(:,1), X(:,2), X(:,3), H, 'edgecolor', 'none');
axis equal; colorbar;
title('Mean curvature');

% Gaussian curvature
K = S(:,1).*S(:,3) - S(:,2).^2;
subplot(1,3,3);
trisurf(T, X(:,1), X(:,2), X(:,3), K, 'edgecolor', 'none');
axis equal; colorbar;
title('Gaussian curvature');
