function Regge = regge_basis(M, X, normal, corner_angle)

vec_norm = @(x) sqrt(sum(x.^2,2));
vec_normalize = @(x) x./sqrt(sum(x.^2,2));

% Local triangle basis
e1r = X(M.T(:,2),:) - X(M.T(:,1),:);
e1r = vec_normalize(e1r);
e2r = cross(normal, e1r, 2);

% Edge tangents in local basis
E1 = X(M.T(:,2),:) - X(M.T(:,1),:);
E2 = X(M.T(:,3),:) - X(M.T(:,2),:);
E3 = X(M.T(:,1),:) - X(M.T(:,3),:);

Regge.edge_length_tri = [vec_norm(E1), vec_norm(E2), vec_norm(E3)];

t1 = [dot(E1, e1r, 2), dot(E1, e2r, 2)];
t2 = [dot(E2, e1r, 2), dot(E2, e2r, 2)];
t3 = [dot(E3, e1r, 2), dot(E3, e2r, 2)];

t1 = vec_normalize(t1);
t2 = vec_normalize(t2);
t3 = vec_normalize(t3);

% Edge normals in local basis
B1 = cross(normal, E1, 2);
B2 = cross(normal, E2, 2);
B3 = cross(normal, E3, 2);

n1 = [dot(B1, e1r, 2), dot(B1, e2r, 2)];
n2 = [dot(B2, e1r, 2), dot(B2, e2r, 2)];
n3 = [dot(B3, e1r, 2), dot(B3, e2r, 2)];

n1 = vec_normalize(n1);
n2 = vec_normalize(n2);
n3 = vec_normalize(n3);

% Regge basis in local coordinate 
% symmetric matrix [a,b;b,c] be like [a,b,c]
sin_ang = sin(corner_angle);

T1 = [2*n2(:,1).*n3(:,1), n2(:,1).*n3(:,2) + n2(:,2).*n3(:,1), 2*n2(:,2).*n3(:,2)];
T1 =-T1./(2*sin_ang(:,1).*sin_ang(:,2));
T2 = [2*n3(:,1).*n1(:,1), n3(:,1).*n1(:,2) + n3(:,2).*n1(:,1), 2*n3(:,2).*n1(:,2)];
T2 =-T2./(2*sin_ang(:,2).*sin_ang(:,3));
T3 = [2*n1(:,1).*n2(:,1), n1(:,1).*n2(:,2) + n1(:,2).*n2(:,1), 2*n1(:,2).*n2(:,2)];
T3 =-T3./(2*sin_ang(:,3).*sin_ang(:,1));

err = max(abs(T1(:) + T2(:) + T3(:) - [ones(M.nf,1); zeros(M.nf,1); ones(M.nf,1)]));
if err > 1e-6
    warning(['Possibly ill-defined triangles: symmetric tensor bases do not sum to identity -- Err : ', num2str(err)]);
end

% Store everything in a structure
Regge.e1r = e1r;
Regge.e2r = e2r;
Regge.e3r = normal;

Regge.t1 = t1;
Regge.t2 = t2;
Regge.t3 = t3;

Regge.n1 = n1;
Regge.n2 = n2;
Regge.n3 = n3;

Regge.T1 = T1;
Regge.T2 = T2;
Regge.T3 = T3;
