function [K,id_sing,sing,e1] = frame_field_vertex_sing(X, T, p, if_plot)

if ~exist('if_plot', 'var')
    if_plot = false;
end
p = round(p);

% Compute stuff
M = MeshInfo(T);
[normal,area] = compute_normals(X, M.T);
corner_angle = angles_of_triangles(X, M.T);
dec = compute_dec_mat(M, corner_angle, area);

% Parallel transport
comp_angle = @(u,v,n) atan2(dot(cross(u,v,2),n,2), dot(u,v,2));
normalize = @(x) x./sqrt(sum(x.^2,2));

edge = X(M.E2V(:,2),:) - X(M.E2V(:,1),:);
edge = normalize(edge);

e1r_face = X(M.T(:,2),:) - X(M.T(:,1),:);
e1r_face = normalize(e1r_face);
e2r_face = cross(normal, e1r_face, 2);

edge_angles = zeros(M.ne,2);
edge_angles(M.ide_int,1) = comp_angle(edge(M.ide_int,:), e1r_face(M.E2T2(M.ide_int,1),:), normal(M.E2T2(M.ide_int,1),:));
edge_angles(M.ide_int,2) = comp_angle(edge(M.ide_int,:), e1r_face(M.E2T2(M.ide_int,2),:), normal(M.E2T2(M.ide_int,2),:));

para_trans = wrapToPi(edge_angles(:,1) - edge_angles(:,2));
para_trans(M.ide_bound) = 0;

% assert(norm(wrapToPi(dec.d1d*para_trans - K)) < 1e-6, 'Gaussian curvature incompatible with angle defect.');

% Frame field laplacian
I = repmat((1:M.ne)', [1,2]);
J = M.E2T2;
S = [exp(p*1i*para_trans),-ones(M.ne,1)];
d0d = sparse(I, J, S, M.ne, M.nf);
W = d0d'*dec.star1d*d0d;
W = (W + W')/2;

% Smoothest frame field
[P,D] = eigs(W, dec.star0d, 5, 'sm');
z = P(:,1);
z = z./abs(z);
dt = 20*real(D(1,1));

for i = 1:20
    z = (W + dt*dec.star0d)\(dt*dec.star0d*z);
    z = z./abs(z);
end

ang = angle(z)/p;
dir = cell(p,1);
for i = 1:p
    angi = ang + (i-1)*2*pi/p;
    dir{i} = cos(angi).*e1r_face + sin(angi).*e2r_face;
end
e1 = dir{1};

% Singularities
omega = wrapToPi(dec.d0d*angle(z) + p*para_trans)/p;
omega(M.ide_bound) = 0;
sing = wrapToPi(dec.d1d*(para_trans - omega))/(2*pi);

assert(max(abs(round(p*sing) - p*sing)) < 1e-6, 'Cannot extract singular vertices.');
sing = round(p*sing)/p;
K = 2*pi*sing;
id_sing = find(abs(sing) > 1/(4*p));
sing = 2*pi*sing(id_sing);

% Plot
if if_plot
    pt = (X(M.T(:,1),:) + X(M.T(:,2),:) + X(M.T(:,3),:))/3;
    figure;
    trisurf(M.T, X(:,1), X(:,2), X(:,3), K, 'edgecolor', 'none', 'facecolor', 'interp');
    hold on; 
    for i = 1:p
        quiver3(pt(:,1), pt(:,2), pt(:,3), dir{i}(:,1), dir{i}(:,2), dir{i}(:,3), 'Color',"#D95319", 'ShowArrowHead','off'); 
    end
    hold off;
    axis equal; colorbar;
    title('Frame field singular points');
end
