function [Kt,id_sing,sing] = polycube_sing(X, T, if_plot)

if ~exist('if_plot', 'var')
    if_plot = false;
end
nv = size(X,1);

% Flagging
normal = compute_normals(X, T);
[~,id_tri] = max(abs(normal), [],2);
id_tri = ((id_tri == 1).*sign(normal(:,1)) + (id_tri == 2).*sign(normal(:,2)) + (id_tri == 3).*sign(normal(:,3))).*id_tri;

% Flagging corners
corner_id = zeros(nv,6);
j = 0;
for i = [-3:-1, 1:3]
    j = j + 1;
    corner_id(:,j) = accumarray(T(:), repmat(id_tri == i, 3,1));
end
id_sing = find(sum(corner_id ~= 0, 2) == 3);

% Cleaning corners
M = MeshInfo(T);
ide = all(ismember(M.E2V, id_sing), 2);
while any(ide)
    id = find(ide,1);
    id_sing = setdiff(id_sing, M.E2V(id,1));
    ide = all(ismember(M.E2V, id_sing), 2);
end

% Corner sign
corner_angle = angles_of_triangles(X, T);
K = 2*pi - accumarray(T(:), corner_angle(:));
sing = (pi/2)*sign(K(id_sing));

Kt = zeros(nv,1);
Kt(id_sing) = sing;

if if_plot
    figure;
    trisurf(T, X(:,1), X(:,2), X(:,3), id_tri);
    axis equal; colorbar;
    title('Flagging');
    
    figure;
    trisurf(T, X(:,1), X(:,2), X(:,3), Kt, 'facecolor', 'interp');
    axis equal; colorbar;
    title('Singular points');
end

assert(abs(sum(K) - sum(Kt)) < 1e-6, 'Cannot extract valid polycube.');
