function [E2V, T2E, E2T, E2T2] = compute_connectivity(T)
% Computes the adjacency properties given a list of triangles
%
% INPUT: 
% T : list of triangles
%
% OUTPUT:
% E2V : list of edges = per of vertices indexes
% T2E : edge indexes per triangles
% E2T : adjacent triangles to each edge , sign of the edge seen from triangle
% T2T : adjacent triangles to each triangle

nf = size(T,1);

E2V = [T(:,1) T(:,2) ; T(:,2) T(:,3) ; T(:,3) T(:,1)];
[E2V,id] = sort(E2V,2);
[E2V,ia,ic] = unique(E2V,'rows');

edgeSg = id(:,1) - id(:,2);
t2es = [edgeSg(1:nf), edgeSg((nf+1):(2*nf)), edgeSg((2*nf+1):end)];
edgeSg = edgeSg(ia);

e1 = ic(1:nf);
e2 = ic((nf+1):(2*nf));
e3 = ic((2*nf+1):end);
T2E = [e1 e2 e3];

bound = find(accumarray(T2E(:), 1) == 1);
[~, idx] = sort([e1; e2; e3; bound]);
idx(idx <= 3*nf) = mod(idx(idx <= 3*nf)-1, nf)+1;
E2T = reshape(idx', [2, size(E2V,1)])';
E2T(E2T > nf) = 0;
E2T = [E2T, edgeSg, -edgeSg];

T2E = T2E.*t2es;

% Edge to triangle with boundaries
E2T2 = zeros(size(E2T,1),2);
E2T2(:,1) = E2T(:,1).*(E2T(:,3) > 0) + E2T(:,2).*(E2T(:,3) < 0);
E2T2(:,2) = E2T(:,1).*(E2T(:,3) < 0) + E2T(:,2).*(E2T(:,3) > 0);
ide = E2T2 ~= 0;
E2T2(~ide(:,1),1) = E2T2(~ide(:,1),2);
E2T2(~ide(:,2),2) = E2T2(~ide(:,2),1);

end