function dec = compute_dec_mat(M, corner_angle, area)

nv = M.nv;
ne = M.ne;
nf = M.nf;

cot_angle = cot(corner_angle);

vor_area = accumarray(M.T(:), [area; area; area]/3);
cotweight = accumarray(abs(M.T2E(:)), vec(cot_angle(:,[3 1 2]))/2);
assert(all(vor_area > 0), 'Negative vertex area.');

if any(cotweight < 1e-5)
    warning('Non Delaunay tet-mesh: risk of convergence issues!');
    % cotweight = max(cotweight, 1e-5); % clamp to avoid problems
end

d0p = sparse([1:ne;1:ne]', M.E2V, [ones(ne,1),-ones(ne,1)], ne, nv);
d1p = sparse([1:nf;1:nf;1:nf]', abs(M.T2E), sign(M.T2E), nf, ne);
assert(norm(d1p*d0p, 'fro') == 0, 'Assembling DEC: Orinetation problems');

star0p = sparse(1:nv, 1:nv, vor_area, nv, nv);
star1p = sparse(1:ne, 1:ne, cotweight, ne, ne);
star2p = sparse(1:nf, 1:nf, 1./area, nf, nf);

d0d = d1p';
d1d = d0p';
assert(norm(d1d*d0d, 'fro') == 0, 'Assembling DEC: Orinetation problems');

star0d = sparse(1:nf, 1:nf, area, nf, nf);
star1d = sparse(1:ne, 1:ne, 1./cotweight, ne, ne);
star2d = sparse(1:nv, 1:nv, 1./vor_area, nv, nv);

% output
dec.d0p = d0p;
dec.d1p = d1p;
dec.d0d = d0d;
dec.d1d = d1d;

dec.star0p = star0p;
dec.star1p = star1p;
dec.star2p = star2p;
dec.star0d = star0d;
dec.star1d = star1d;
dec.star2d = star2d;
dec.cotweight = cotweight;

% Triangle based operator
W = d0p'*star1p*d0p;
W = (W + W')/2;
dec.W = W;

end
