function [vf1,vf2,vf3] = primal_harmonic_form_vector_field(M, X, area, hp, if_plot)
% From "Robust Fairing via Conformal Curvature Flow"
% Keenan Crane, Ulrich Pinkall and Peter Schröder

% Compute face-based vector field associated to primal harmonic form

if ~exist('if_plot', 'var')
    if_plot = false;
end

nb_harmonics = size(hp,2);

vf1 = zeros(M.nf,nb_harmonics);
vf2 = zeros(M.nf,nb_harmonics);
vf3 = zeros(M.nf,nb_harmonics);
if nb_harmonics > 0
    hp_tri1 = hp(abs(M.T2E(:,1)),:).*sign(M.T2E(:,1));
    hp_tri2 = hp(abs(M.T2E(:,2)),:).*sign(M.T2E(:,2));
    e1 = X(M.T(:,2),:) -  X(M.T(:,1),:);
    e2 = X(M.T(:,3),:) -  X(M.T(:,2),:);
    a = ( hp_tri1.*dot(e2, e2, 2) - hp_tri2.*dot(e1, e2, 2))./(2*area).^2;
    b = (-hp_tri1.*dot(e1, e2, 2) + hp_tri2.*dot(e1, e1, 2))./(2*area).^2;
    vf1 = a.*e1(:,1) + b.*e2(:,1);
    vf2 = a.*e1(:,2) + b.*e2(:,2);
    vf3 = a.*e1(:,3) + b.*e2(:,3);
end

if if_plot
    for i = 1:nb_harmonics
        pt = (X(M.T(:,1),:) + X(M.T(:,2),:) + X(M.T(:,3),:))/3;
        figure;
        trisurf(M.T, X(:,1), X(:,2), X(:,3), 1);
        hold on; quiver3(pt(:,1), pt(:,2), pt(:,3), vf1(:,i), vf2(:,i), vf3(:,i)); hold off;
        axis equal; view(0,90);
    end
end