function h = primal_harmonic_one_form_basis(M, dec)
% From "Robust Fairing via Conformal Curvature Flow"
% Keenan Crane, Ulrich Pinkall and Peter Schröder

chi = M.nf - M.ne + M.nv;
if ~M.is_bnd
    nb_harmonics = 2 - chi;
else
    nb_harmonics = 2 - chi - 1;
end

if nb_harmonics > 0
    h = randn(M.ne, nb_harmonics);

    Wp = dec.d0p'*dec.star1p*dec.d0p;
    f = Wp\(dec.d0p'*dec.star1p*h);

    Wd = dec.d0d'*dec.star1d*dec.d0d;
    g = Wd\(dec.d0d'*h);

    h = h - dec.d0p*f - dec.star1d*dec.d0d*g;

    h = orthonormalize(h, dec.star1p);
else
    h = zeros(M.ne,nb_harmonics);
end

end

function X = orthonormalize(X, A)
n = size(X,2);

X(:,1) = X(:,1)/sqrt(X(:,1)'*A*X(:,1));
for i = 2:n
    for j = 1:i-1
        X(:,i) = X(:,i) - (X(:,i)'*A*X(:,j))*X(:,j);
    end

    X(:,i) = X(:,i)/sqrt(X(:,i)'*A*X(:,i));
end
end

