function [Xn,Tn] = subdivide_triangle(X, T, ifloop)
% Loop subdivision

if ~exist('ifloop','var')
    ifloop = false;
end

[E2V, T2E, E2T] = compute_connectivity(T);

[~,t2e12] = ismember(sort([T(:,1) T(:,2)], 2), E2V, 'rows');
[~,t2e23] = ismember(sort([T(:,2) T(:,3)], 2), E2V, 'rows');
[~,t2e31] = ismember(sort([T(:,3) T(:,1)], 2), E2V, 'rows');

nv = size(X,1);
ne = size(E2V,1);
nf = size(T,1);

Xn = zeros(nv+ne,3);
if ~ifloop
    Xn(1:nv,:) = X;
    Xn(nv+1:end,:) = (X(E2V(:,1),:) + X(E2V(:,2),:))/2;
else
    ide_bnd = any(E2T(:,1:2) == 0, 2);
    ide_int =~ide_bnd;
    idx_bnd = unique(E2V(ide_bnd,:));

    val = accumarray(T(:), 1);
    beta = 3./(8*val);
    beta(val == 3) = 3/16;
    A = sparse(T(:,[1 2 3 1 2 3 1 2 3]), T(:,[2 3 1 3 1 2 1 2 3]), [beta(T(:,[1 2 3 1 2 3]))/2, 1./val(T(:,[1 2 3])) - beta(T(:,[1 2 3]))], nv, nv);
    Xn(1:nv,:) = A*X;
    B = sparse(E2V(ide_bnd,[1 2 1 2]), E2V(ide_bnd,[2 1 1 2]), [ones(sum(ide_bnd),2)/8, ones(sum(ide_bnd),2)*3/8], nv, nv);
    Xb = B*X;
    Xn(idx_bnd,:) = Xb(idx_bnd,:);

    idx_adj = [T(E2T(ide_int,1),:), T(E2T(ide_int,2),:)];
    Xe = zeros(ne,3);
    Xe(ide_bnd,:) = (X(E2V(ide_bnd,1),:) + X(E2V(ide_bnd,2),:))/2;
    Xe(ide_int,:) = (X(E2V(ide_int,1),:) + X(E2V(ide_int,2),:))/8 + [sum(reshape(X(idx_adj,1), [],6),2), sum(reshape(X(idx_adj,2), [],6),2), sum(reshape(X(idx_adj,3), [],6),2)]/8;

    Xn(nv+1:end,:) = Xe;
end

Tn = [T(:,1), nv+t2e12, nv+t2e31; ...
      T(:,2), nv+t2e23, nv+t2e12; ...
      T(:,3), nv+t2e31, nv+t2e23; ...
      nv+t2e12, nv+t2e23, nv+t2e31];